import setuptools

setuptools.setup(
    name="sh2d",
    version="2025.07.16",
    author="sh2d",
    author_email="xx@qq.com",
    description="",
    long_description='',
    long_description_content_type="text/markdown",
    url="https://github.com/",
    packages=setuptools.find_packages(),
    install_requires=[
        'colorlog',
        'ddddocr',
        'docxtpl',
        'dnspython',
        'gmssl',
        'loguru',
        'openpyxl',
        'paramiko',
        'pycryptodome',
        'pyexecjs2',
        'pymysql',
        'python-docx',
        'python-whois',
        'pyttsx3',
        'pywin32',
        'pyyaml',
        'pyzipper',
        'requests',
        'ruamel.yaml',
        'werkzeug',
        'xlwings',
        'xlrd==1.2.0',
        'xlsxwriter',
        'xlwt',
        'zmail',
        ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
